from peewee import CharField, Check, DateField, FloatField, ForeignKeyField, Model

from sdmcli.db import get_db

db = get_db()


def db_conn(func):
    def inner(*args, **kwargs):
        db.connect()
        db.create_tables([Ledger, Category, Item])
        func(*args, **kwargs)
        db.close()

    return inner


class BaseModel(Model):
    class Meta:
        database = db


class Ledger(BaseModel):
    name = CharField(
        max_length=32, index=True, unique=True, constraints=[Check("length(name) > 0")]
    )


class Category(BaseModel):
    ledger = ForeignKeyField(Ledger, backref="categories", on_delete="CASCADE")
    name = CharField(max_length=32, index=True, constraints=[Check("length(name) > 0")])

    class Meta:
        indexes = ((("ledger", "name"), True),)

    @classmethod
    def get_unique(cls, ledger_name, name):
        return (
            cls.select()
            .join(Ledger)
            .where((Ledger.name == ledger_name) & (cls.name == name))
            .get()
        )


class Item(BaseModel):
    ledger = ForeignKeyField(Ledger, backref="items", on_delete="CASCADE")
    date = DateField()
    type = CharField(max_length=8, constraints=[Check("type == 'expense' OR type == 'income'")])
    description = CharField(max_length=128, constraints=[Check("length(description) > 0")])
    value = FloatField(constraints=[Check("value >= 0")])
    category = ForeignKeyField(Category, backref="items", null=True)
